@@ -22,6 +22,8 @@ from typing import (
 )
 
 from rx.core.typing import Observable
+from PIL import Image
+import numpy as np
 
 
 Callback = Union[
@@ -295,17 +297,43 @@ class ObjectDetection(BaseDetection):
     boundingBox: Tuple[int, int, int, int]
 
 
-class BaseState(TypedDict):
+class BaseStateLight(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["LightStateWithoutLastEvent"]]
+
+
+class BaseStateMotion(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["MotionStateWithoutLastEvent"]]
+
+
+class BaseStateAudio(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["AudioStateWithoutLastEvent"]]
+
+
+class BaseStateObject(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["ObjectStateWithoutLastEvent"]]
+
+
+class BaseStateDoorbell(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["DoorbellStateWithoutLastEvent"]]
+
+
+class BaseStateSiren(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["SirenStateWithoutLastEvent"]]
+
+
+class BaseStateBattery(TypedDict):
+    timestamp: int
+    lastEvent: NotRequired[Optional["BatteryStateWithoutLastEvent"]]
+
+
+class BaseStateWithoutLastEvent(TypedDict):
     timestamp: int
-    lastEvent: Union[
-        "LightState",
-        "MotionState",
-        "AudioState",
-        "ObjectState",
-        "DoorbellState",
-        "SirenState",
-        "BatteryState",
-    ]
 
 
 class MotionSetEvent(TypedDict):
@@ -341,36 +369,47 @@ class BatterySetEvent(TypedDict):
     lowBattery: NotRequired[Optional[bool]]
 
 
-class LightState(BaseState, LightSetEvent): ...
+class LightState(BaseStateLight, LightSetEvent): ...
 
 
-class MotionState(BaseState, MotionSetEvent): ...
+class LightStateWithoutLastEvent(BaseStateWithoutLastEvent, LightSetEvent): ...
 
 
-class AudioState(BaseState, AudioSetEvent): ...
+class MotionState(BaseStateMotion, MotionSetEvent): ...
 
 
-class DoorbellState(BaseState, DoorbellSetEvent): ...
+class MotionStateWithoutLastEvent(BaseStateWithoutLastEvent, MotionSetEvent): ...
 
 
-class SirenState(BaseState, SirenSetEvent): ...
+class AudioState(BaseStateAudio, AudioSetEvent): ...
 
 
-class ObjectState(BaseState, ObjectSetEvent): ...
+class AudioStateWithoutLastEvent(BaseStateWithoutLastEvent, AudioSetEvent): ...
 
 
-class BatteryState(BaseState, BatterySetEvent): ...
+class DoorbellState(BaseStateDoorbell, DoorbellSetEvent): ...
 
 
-State = Union[
-    LightState,
-    MotionState,
-    AudioState,
-    DoorbellState,
-    SirenState,
-    ObjectState,
-    BatteryState,
-]
+class DoorbellStateWithoutLastEvent(BaseStateWithoutLastEvent, DoorbellSetEvent): ...
+
+
+class SirenState(BaseStateSiren, SirenSetEvent): ...
+
+
+class SirenStateWithoutLastEvent(BaseStateWithoutLastEvent, SirenSetEvent): ...
+
+
+class ObjectState(BaseStateObject, ObjectSetEvent): ...
+
+
+class ObjectStateWithoutLastEvent(BaseStateWithoutLastEvent, ObjectSetEvent): ...
+
+
+class BatteryState(BaseStateBattery, BatterySetEvent): ...
+
+
+class BatteryStateWithoutLastEvent(BaseStateWithoutLastEvent, BatterySetEvent): ...
+
 
 T = TypeVar(
     "T",
@@ -394,6 +433,16 @@ class StateValues(Protocol):
     battery: BatteryState
 
 
+class SetValues(Protocol):
+    light: LightSetEvent
+    motion: MotionSetEvent
+    audio: AudioSetEvent
+    object: ObjectSetEvent
+    doorbell: DoorbellSetEvent
+    siren: SirenSetEvent
+    battery: BatterySetEvent
+
+
 CameraStateCallbacks = Dict[str, Callable[[T], None]]
 
 StateValue = Union[
@@ -406,7 +455,7 @@ StateValue = Union[
     BatteryState,
 ]
 
-OnSetEvent = Union[
+SetValue = Union[
     LightSetEvent,
     MotionSetEvent,
     AudioSetEvent,
@@ -510,15 +559,16 @@ class Frame(TypedDict):
 
 
 class VideoFrame(Protocol):
-    metadata: FrameMetadata
-    image: Any
-
+    @property
+    def metadata(self) -> FrameMetadata: ...
     def input_width(self) -> int: ...
     def input_height(self) -> int: ...
     def input_format(self) -> str: ...
-    def to_buffer(self, options: Optional[ImageOptions] = None) -> bytes: ...
-    def to_numpy(self, options: Optional[ImageOptions] = None) -> Any: ...
-    def to_image(self, options: Optional[ImageOptions] = None) -> Any: ...
+    async def to_buffer(self, options: Optional[ImageOptions] = None) -> bytes: ...
+    async def to_numpy(
+        self, options: Optional[ImageOptions] = None
+    ) -> np.ndarray[Any, Any]: ...
+    async def to_image(self, options: Optional[ImageOptions] = None) -> Image.Image: ...
 
 
 class CameraDevice(Protocol):
@@ -909,4 +959,4 @@ class BasePlugin(ABC):
         self, action_id: str, payload: Any
     ) -> Dict[Literal["toastMessage"], str]: ...
     @abstractmethod
-    async def configure_cameras(self, cameras: List[CameraDevice]) -> None: ...
+    def configure_cameras(self, cameras: List[CameraDevice]) -> None: ...
